/**
 * Show Unix Timestamp in Human Readable Format as editor hint.
 *
 * Can also convert an editor selection of a Unix Timestamp into Human Readable Format.
 *
 * @category  WeBuilder Plugin
 * @package   Unix Timestamp
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2016 Peter Klein
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   1.02
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     35   function ConvertFromUnixDate()
 *     64   function OnShowHint(CodeType, Hint, AKey, AllowHint)
 *     81   function GetWordByPos(str, pos)
 *    105   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 4
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

// Constant. Sets UnixStartDate to TDateTime of 01/01/1970
var UnixStartDate = 25569; // StrToDate("01/01/1970");

/**
 * Convert Unix timestamp selection into human readable format and insert result in editor
 *
 * @return void
 */
function ConvertFromUnixDate() {
    var sel = Editor.SelText;
    var uSel = Editor.Selection;
    if (Trim(sel) == "") {
        // No selection, so grab word at cursor position
        var res = GetWordByPos(Editor.LinesAsDisplayed[uSel.SelStartLine], uSel.SelStartColReal);
        if ((Length(res[0]) == 10) || (ValidInt(res[0]))) {
            // Set selection on word found
            uSel.SelStartCol = res[1];
            uSel.SelEndCol = res[1] + Length(res[0]);
            Editor.Selection = uSel;
            sel = res[0];
        }
    }
    if ((Length(sel) == 10) || (ValidInt(sel))) {
        Editor.SelText = FormatDateTime(Script.ReadSetting("DateTime format", "dd/mm/yy hh:nn"), UnixStartDate + (StrToFloat(sel) / 86400));
    }
}

/**
 * Convert Unix timestamp selection into human readable format and show as editor hint
 *
 * @param  integer      CodeType
 * @param  string       Hint
 * @param  string       AKey
 * @param  boolean      AllowHint
 *
 * @return void
 */
function OnShowHint(CodeType, Hint, AKey, AllowHint) {
    var res = GetWordByPos(Editor.LinesAsDisplayed[Editor.Selection.SelStartLine], Editor.Selection.SelStartColReal);
    var word = res[0];
    if ((Length(word) == 10) && (ValidInt(word))) {
        Hint = FormatDateTime(Script.ReadSetting("DateTime format", "dd/mm/yy hh:nn"), UnixStartDate + (StrToFloat(word) / 86400));
        AllowHint = true;
    }
}

/**
 * Find word at cursor position
 *
 * @param  string   str: The editor line
 * @param  integer   pos: The cursor position
 *
 * @return array    [word:string, start:integer]
 */
function GetWordByPos(str, pos) {
    var res = "";
    var start = pos;
    if (RegexMatchAll(Copy(str, 0, pos), "([\\w]*)$", true, match, p) == true) {
        // Word part on left side of cursor position
        if (Length(match)) {
            res = _v(match, [0, 1]);
            start -= Length(res);
        }
    }
    if (RegexMatchAll(Copy(str, pos + 1, Length(str) - pos), "^([\\w]*)", true, match, p) == true) {
        // Word part on right side of cursor position
        if (Length(match)) {
            res += _v(match, [0, 1])
        }
    }
    return [_t(res), start];
}

/**
 * Show info when plugin is installed.
 *
 * @return void
 */
function OnInstalled() {
  Alert("Unix Timestamp 1.02 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");
Script.ConnectSignal("show_hint", "OnShowHint");

var bmp = new TBitmap;
var icon = LoadFileToBitmap(Script.Path + "UnixTimestamp.png", bmp);
var act = Script.RegisterDocumentAction("", "Unix Timestamp to date/time", "", "ConvertFromUnixDate");
Actions.SetIcon(act, bmp);
delete bmp;
